class User < ApplicationRecord
  devise :database_authenticatable, :registerable, :rememberable
  has_many :assignments
  has_many :candidates

  validates :password, presence: true, length: { minimum: 6 }, allow_nil: true
  validates :name, uniqueness: { case_sensitive: false, message: "already in use" }, allow_nil: false
  validates :email, uniqueness: { case_sensitive: false, message: "already in use" }, allow_nil: true, allow_blank: true

  before_validation :cleanup_languages
  validates :languages_from, format: { with: /\A([a-z][a-z])(,[a-z][a-z])*\z/, message: "please use comma-separated two-letter codes"}, allow_blank: true
  validates :languages_from, length: { maximum: 14 }

  validates :languages_to, format: { with: /\A([a-z][a-z])(,[a-z][a-z])*\z/, message: "please use comma-separated two-letter codes"}, allow_blank: true
  validates :languages_to, length: { maximum: 14 }

  validates :invitation_token, presence: true, on: :create
  validate :valid_invitation_token, on: :create

  after_initialize :set_avatar_color

  # Custom method to allow user to sign-in with any case while still storing the "correct" case in the database
  def self.find_for_database_authentication(warden_conditions)
    conditions = warden_conditions.dup
    if (login = conditions.delete(:name))
      where(conditions.to_h).where(["lower(name) = :value", { value: login.downcase }]).first
    else
      Rails.logger.warn("Authentication did not query :name as expected, login will only work with exact case!")
      where(conditions.to_h).first
    end
  end

  def self.leaderboard
    all.map { |u| [u.name, u.workload_minutes] }
      .sort_by { |_, workload| -workload }
      .reject { |_, workload| workload.zero? }
      .to_h
  end

  class Session < ApplicationRecord
    has_many :assignments

    def workload_minutes
      # Logic to calculate workload in minutes
      60 # This is just a placeholder
    end
  end


  def errors
    super.tap { |errors| errors.delete(:password, :blank) if password.nil? }
  end

  def has_password?
    !password.nil?
  end

  def text_color
    r, g, b = avatar_color.delete_prefix('#').chars.each_slice(2).map { |hex| hex.join.to_i(16) }

    # Calculate relative luminance (WCAG 2.0 formula)
    luminance = (0.2126 * r + 0.7152 * g + 0.0722 * b) / 255

    # Choose text color based on luminance threshold
    luminance > 0.5 ? "#000000" : "#ffffff"
  end

  def initials
    name.split(/\s+/).map(&:first).join('')
  end

  def set_avatar_color
    return unless self.avatar_color.nil?

    r = rand(256)
    g = rand(256)
    b = rand(256)

    # r = [r, 128].max
    # g = [g, 128].max
    # b = [b, 128].max

    self.avatar_color = "##{r.to_s(16).rjust(2, '0')}#{g.to_s(16).rjust(2, '0')}#{b.to_s(16).rjust(2, '0')}"
  end

  def workload_minutes
    Assignment.includes(:session).where(user: self).sum { | a | a.session.duration_minutes }
  end

  private

  def valid_invitation_token
    valid_tokens = ["gargamel"]
    errors.add(:invitation_token, "is invalid") unless valid_tokens.include?(invitation_token)
  end

  def cleanup_languages
    self.languages_from = self.languages_from&.gsub(/\s+/, '')&.downcase
    self.languages_to = self.languages_to&.gsub(/\s+/, '')&.downcase
  end
end
